using System;
using SWF = System.Windows.Forms;
using ST = System.Threading;
using T = System.Diagnostics.Trace;

namespace BankCustomerApp
{

	public class App
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		public static void Main() 
		{
			try
			{
				// setup generic exception handler...
				SWF.Application.ThreadException += new ST.ThreadExceptionEventHandler(App.OnAppException);

				// setup cleanup routine for normal app shutdown...
				SWF.Application.ThreadExit += new System.EventHandler(App.OnAppExit);

				// when app starts, delete the previous log file (if any)
				string  logfile = System.AppDomain.CurrentDomain.BaseDirectory + "AppLog.txt";
				System.IO.TextWriter  log = new System.IO.StreamWriter(logfile);

				if (Globals.Trace)  // then also send trace output to log file
				{
					System.Diagnostics.TextWriterTraceListener  logger;
					logger = new System.Diagnostics.TextWriterTraceListener(log);
					System.Diagnostics.Trace.Listeners.Add(logger);
					System.Diagnostics.Trace.WriteLine("App starting: " + DateTime.Now);
				}

				SWF.Application.Run(new Form1());
			}
			catch(Exception ex)
			{
				OnAppException(null, new ST.ThreadExceptionEventArgs(ex));
			}
		}


		/// <summary>
		/// Generic exception handler for application.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private static void OnAppException(object sender, ST.ThreadExceptionEventArgs e)
		{
			Exception  ex;
			ex = e.Exception;

			// inform user...
			SWF.MessageBox.Show("Halting due to error: " + ex.Message, "Bank Customer App", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);

			// log it (the entire exception chain)...
			T.WriteLine("Generic Application Exception Handler:");
			while (ex != null)
			{
				T.WriteLine(ex.GetType().FullName);
				T.Indent();
				T.WriteLine("Msg: " + ex.Message);
				T.WriteLine("Trace: " + ex.StackTrace);
				T.Unindent();
				T.Flush();

				ex = ex.InnerException;
			}//while

			// halt app
			T.Close();
			SWF.Application.Exit();
		}

		/// <summary>
		/// Called during normal application termination.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private static void OnAppExit(object sender, System.EventArgs e)
		{
			T.Close();  // close trace file
		}

	}//class
}//namespace
