using System;
using System.Data;
using SQL = System.Data.SqlClient;
using MSA = System.Data.OleDb;

namespace DataAccessTier
{

	public class DataAccess
	{
		private string  m_Connection;

		//
		// factory methods for creating vendor-specific classes
		//
		private IDbConnection CreateConnection(string connection)
		{ 
			// SQL Server:
			return new SQL.SqlConnection(connection);

			// OleDB (e.g. MS Access):
			// return new MSA.OleDbConnection(connection);
		}

		private IDbCommand CreateCommand(string sql, IDbConnection conn)
		{ 
			// SQL Server:
			return new SQL.SqlCommand(sql, (SQL.SqlConnection) conn);

			// OleDB (e.g. MS Access):
			// return new MSA.OleDbCommand(sql, (MSA.OleDbConnection) conn);
		}

		private IDbDataAdapter CreateAdapter(IDbCommand cmd)
		{ 
			// SQL Server:
			return new SQL.SqlDataAdapter((SQL.SqlCommand) cmd);

			// OleDB (e.g. MS Access):
			// return new MSA.OleDbDataAdapter((MSA.OleDbCommand) cmd);
		}

		//
		// constructs object based on database connection string
		//
		public DataAccess(string sConnection)
		{
			this.m_Connection = sConnection;
		}

		//
		// generic method to retrieve records into a dataset, returning dataset...
		//
		public System.Data.DataSet Retrieve(string sql)
		{
			IDbConnection   conn = null;
			IDbCommand      cmd;
			IDbDataAdapter  adapter;
			DataSet         ds = new DataSet();

			try
			{
				conn    = CreateConnection(this.m_Connection);
				cmd     = CreateCommand(sql, conn);
				adapter = CreateAdapter(cmd);

				conn.Open();
				adapter.Fill(ds);
			}
			catch(Exception ex)
			{
				throw ex;
			}
			finally 
			{
				try{ conn.Close(); } 
				catch{}
			}

			return ds;
		}

		//
		// generic method to delete a record based on given sql...
		//
		public void Delete(string sql)
		{
			IDbConnection   conn = null;
			IDbCommand      cmd;

			try
			{
				conn    = CreateConnection(this.m_Connection);
				cmd     = CreateCommand(sql, conn);

				int  rows;
				
				conn.Open();
				rows = cmd.ExecuteNonQuery();
				conn.Close();

				if (rows != 1)
					throw new System.ApplicationException("Delete failed?!");
			}
			catch(Exception ex)
			{
				throw ex;
			}
			finally 
			{
				try{ conn.Close(); } 
				catch{}
			}
		}

	}//class
}//namespace
